/* -*-mode:C++-*- */
/*
 * ksi.h
 * ksi public interface
 *
 * Copyright (C) 1997-2001, ivan demakov
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without a written agreement
 * is hereby granted, provided that the above copyright notice and this
 * paragraph and the following two paragraphs appear in all copies.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE AUTHORS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *
 * Author:        Ivan Demakov <demakov@users.sourceforge.net>
 * Creation date: Thu Jul 31 21:49:27 1997
 * Last Update:   Sat Sep 15 17:43:08 2001
 *
 *
 * $Id: ksi.h,v 1.25 2001/09/15 12:46:33 demakov Exp $
 *
 */


#ifndef KSI_H
#define KSI_H

#if defined(__WATCOMC__)
#  if !defined(MSWIN32) && !defined(OS2) && !defined(DOS4G)
#    if defined(__NT__)
#      define MSWIN32
#    elif defined(__OS2__)
#      define OS2
#    elif defined(MSDOS)
#      define DOS4G
#    endif
#  endif
#endif

#if defined(_MSC_VER)
#  if !defined(MSWIN32)
#    define MSWIN32
#  endif
#endif


#ifdef KSI_IMPL_DEFS
#  include <ksi_conf.h>
#else
#  include "ksi_ver.h"
#  ifdef HAVE_CONFIG_H
#    include <config.h>
#  elif defined(__STDC__) || defined(_MSC_VER) || defined(__WATCOMC__) || defined(__GNUC__)
#    define HAVE_STDARG_H 1
#    define HAVE_STDDEF_H 1
#    define HAVE_STDLIB_H 1
#    define HAVE_STRING_H 1
#  endif
#endif

#ifdef HAVE_STDARG_H
#  include <stdarg.h>
#endif
#ifdef HAVE_STDDEF_H
#  include <stddef.h>
#endif
#ifdef HAVE_STDLIB_H
#  include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#  include <string.h>
#endif
#ifdef HAVE_MEMORY_H
#  include <memory.h>
#endif


#if !defined(KSI_INT_TYPE) && defined(SIZEOF_VOIDP) && defined(SIZEOF_INT) && defined(SIZEOF_LONG)
#  if SIZEOF_VOIDP == SIZEOF_INT
#    define KSI_INT_TYPE		int
#    define KSI_SIZEOF_INT_TYPE		SIZEOF_INT
#  elif SIZEOF_VOIDP == SIZEOF_LONG
#    define KSI_INT_TYPE		long
#    define KSI_SIZEOF_INT_TYPE		SIZEOF_LONG
#  endif
#endif

#ifndef KSI_INT_TYPE
#  if defined(_MSC_VER) || defined(__WATCOMC__)
#    define KSI_INT_TYPE		int
#    define KSI_SIZEOF_INT_TYPE		SIZEOF_INT
#  elif defined(i386) || defined(__i386__)
#    define KSI_INT_TYPE		int
#    define KSI_SIZEOF_INT_TYPE		SIZEOF_INT
#  elif defined(__alpha) || defined(__alpha__)
#    define KSI_INT_TYPE		long
#    define KSI_SIZEOF_INT_TYPE		SIZEOF_LONG
#  else
#    error "unknown integer type"
#  endif
#endif

#ifndef LONG_LONG
#  if defined(__GNUC__)
#    define LONG_LONG long long
#  elif defined(_MSC_VER)
#    define LONG_LONG __int64
#  endif
#endif


#if defined(__WATCOMC__)
#  if defined(MSWIN32)
#    if defined(KSIDLL)
#      if defined(MAKE_si_LIB)
#        define SI_API extern __declspec(dllexport)
#        define SI_DATA extern __declspec(dllexport)
#      else
#        define SI_API extern __declspec(dllimport)
#        define SI_DATA extern __declspec(dllimport)
#      endif
#    endif
#  endif
#endif

#if defined(_MSC_VER)
#  if defined(KSIDLL)
#    if defined(MAKE_si_LIB)
#      define SI_API __declspec(dllexport)
#      define SI_DATA __declspec(dllexport)
#    else
#      define SI_API __declspec(dllimport)
#      define SI_DATA __declspec(dllexport)
#    endif
#  endif
#endif

#ifndef SI_API
#  define SI_API extern
#endif

#ifndef SI_DATA
#  define SI_DATA extern
#endif

#ifndef KSI_STACK_ELEM
# define KSI_STACK_ELEM KSI_INT_TYPE
#endif


/* Forward declaration of Ksi objects */

struct Ksi_Tag;
struct Ksi_Port_Tag;
struct Ksi_Event_Tag;
struct Ksi_Event_Mgr;
struct Ksi_Valtab;
struct Ksi_Obj;
struct Ksi_ExtObj;
struct Ksi_Symbol;
struct Ksi_Keyword;
struct Ksi_Prim;
struct Ksi_Pair;
struct Ksi_Port;
struct Ksi_Bignum;
struct Ksi_Rational;
struct Ksi_Flonum;
struct Ksi_Complex;
struct Ksi_String;
struct Ksi_Vector;
struct Ksi_Environ;
struct Ksi_Jump;
struct Ksi_Wind;
struct Ksi_Closure;
struct Ksi_Promise;
struct Ksi_Pointer;
struct Ksi_Code;
struct Ksi_Frame;
struct Ksi_Values;
struct Ksi_Hashtab;
struct Ksi_Instance;
struct Ksi_NextMethod;
struct Ksi_Event;

typedef struct Ksi_Tag		*ksi_tag;
typedef struct Ksi_Port_Tag	*ksi_port_tag;
typedef struct Ksi_Event_Tag	*ksi_event_tag;
typedef struct Ksi_Event_Mgr	*ksi_event_mgr;
typedef struct Ksi_Valtab	*ksi_valtab;
typedef struct Ksi_Obj		*ksi_obj;
typedef struct Ksi_ExtObj	*ksi_extobj;
typedef struct Ksi_Symbol	*ksi_symbol;
typedef struct Ksi_Keyword	*ksi_keyword;
typedef struct Ksi_Prim		*ksi_prim;
typedef struct Ksi_Pair		*ksi_pair;
typedef struct Ksi_Port		*ksi_port;
typedef struct Ksi_Bignum	*ksi_bignum;
typedef struct Ksi_Rational	*ksi_rational;
typedef struct Ksi_Flonum	*ksi_flonum;
typedef struct Ksi_Complex	*ksi_complex;
typedef struct Ksi_String	*ksi_string;
typedef struct Ksi_Vector	*ksi_vector;
typedef struct Ksi_Environ	*ksi_env;
typedef struct Ksi_Jump		*ksi_jump;
typedef struct Ksi_Wind		*ksi_wind;
typedef struct Ksi_Closure	*ksi_closure;
typedef struct Ksi_Promise	*ksi_promise;
typedef struct Ksi_Pointer	*ksi_pointer;
typedef struct Ksi_Code		*ksi_code;
typedef struct Ksi_Frame	*ksi_frame;
typedef struct Ksi_Values	*ksi_values;
typedef struct Ksi_Hashtab	*ksi_hashtab;
typedef struct Ksi_Instance	*ksi_instance;
typedef struct Ksi_NextMethod	*ksi_next_method;
typedef struct Ksi_Event	*ksi_event;


/* 2-   . */
enum ksi_tc2
{
  KSI_TC2_PTR = 0,
  KSI_TC2_INT = 1,
  KSI_TC2_IMM = 3
};

/* 8-    */
enum ksi_tc8
{
  KSI_TC8_CHAR		= KSI_TC2_IMM,
  KSI_TC8_BOOL		= KSI_TC2_IMM | (1 << 2),
  KSI_TC8_EXIT		= KSI_TC2_IMM | (2 << 2),
  KSI_TC8_VOID		= KSI_TC2_IMM | (3 << 2),
  KSI_TC8_EOF		= KSI_TC2_IMM | (4 << 2),
  KSI_TC8_NIL		= KSI_TC2_IMM | (5 << 2),
  KSI_TC8_ERR		= KSI_TC2_IMM | (6 << 2),
  KSI_TC8_ANY		= KSI_TC2_IMM | (7 << 2),

  KSI_TC8_LAST		= KSI_TC2_IMM | (63 << 2)
};

enum ksi_errlog_priority_t
{
  ERRLOG_EMERG = 0,		/* system is unusable */
  ERRLOG_ALERT,			/* action must be taken immediately */
  ERRLOG_CRIT,			/* critical conditions */
  ERRLOG_ERROR,			/* error conditions */
  ERRLOG_WARNING,		/* warning conditions */
  ERRLOG_NOTICE,		/* normal but significant condition */
  ERRLOG_INFO,			/* informational */
  ERRLOG_DEBUG,			/* debug-level messages */
  ERRLOG_ALL = ERRLOG_DEBUG
};


/* Some typedefs */

typedef KSI_STACK_ELEM*         ksi_stack;

typedef void (*ksi_finalizer_function) (void* obj, void* client_data);

typedef void (*ksi_init_function) (void* client_data);


/* Some macros */

#define KSI_PTR(c)	((ksi_obj) (c))
#define KSI_COD(c)	((KSI_INT_TYPE) (c))

#define KSI_MK_TC8(c,t)	(KSI_PTR ((KSI_COD (c) << 8) | (t)))
#define KSI_TC8_P(c,t)	((KSI_COD ((c)) & 0xff) == (t))
#define KSI_TC8_COD(c)	(KSI_COD ((c)) >> 8)
#define KSI_TC8_TAG(c)	(KSI_COD ((c)) & 0xff)

/*
 * KSI_PTR_P	pointer?
 * KSI_IMM_P	imm value?
 * KSI_OBJ_P	valid pointer?
 */

#define KSI_PTR_P(c)	((KSI_COD ((c)) & 1) == 0)
#define KSI_IMM_P(c)	((KSI_COD ((c)) & 1) != 0)
#if !defined(CAUTIOUS) || CAUTIOUS >= 2
#define KSI_OBJ_P(c)	(KSI_PTR_P ((c)) && (c) != 0)
#else
#define KSI_OBJ_P(c)	(KSI_PTR_P ((c)))
#endif

/*
 * KSI_SINT_P	ksi integer?
 * KSI_MK_SINT	make ksi integer from C int
 * KSI_SINT_COD	get C int from ksi integer
 */

#define KSI_SINT_P(c)	((KSI_COD (c) & 3) == KSI_TC2_INT)
#define KSI_MK_SINT(c)	(KSI_PTR ((KSI_COD (c) << 2) | KSI_TC2_INT))
#define KSI_SINT_COD(c)	(KSI_COD (c) >> 2)

/*
 * KSI_CHAR_P	ksi char?
 * KSI_MK_CHAR	make ksi char from C char
 * KSI_CHAR_COD	get C char from ksi object
 */

#define KSI_CHAR_P(c)	KSI_TC8_P ((c), KSI_TC8_CHAR)
#define KSI_MK_CHAR(c)	KSI_MK_TC8 (((c) & 0xff), KSI_TC8_CHAR)
#define KSI_CHAR_COD(x)	KSI_TC8_COD (x)

/*
 * KSI_BOOL_P	ksi bool?
 * KSI_MK_BOOL	make ksi bool from C int
 * KSI_BOOL_COD get C int from ksi bool
 */

#define KSI_BOOL_P(x)	KSI_TC8_P ((x), KSI_TC8_BOOL)
#define KSI_MK_BOOL(x)	KSI_MK_TC8 ((x), KSI_TC8_BOOL)
#define KSI_BOOL_COD(x)	KSI_TC8_COD ((x))


/*
 * KSI_EXIT_P	ksi exit-object?
 * KSI_MK_EXIT	make ksi exit-code from C int
 * KSI_EXIT_COD	get C int from ksi exit-code
 */

#define KSI_EXIT_P(c)	KSI_TC8_P ((c), KSI_TC8_EXIT)
#define KSI_MK_EXIT(c)	KSI_MK_TC8 ((c), KSI_TC8_EXIT)
#define KSI_EXIT_COD(c)	KSI_TC8_COD ((c))


/* some imm objects */

#define ksi_null	((ksi_obj) 0)
#define ksi_void	KSI_MK_TC8 (0, KSI_TC8_VOID)

#define ksi_false	KSI_MK_BOOL (0)
#define ksi_true	KSI_MK_BOOL (1)

#define ksi_zero	KSI_MK_SINT (0)
#define ksi_one		KSI_MK_SINT (1)
#define ksi_two		KSI_MK_SINT (2)
#define ksi_three	KSI_MK_SINT (3)
#define ksi_four	KSI_MK_SINT (4)
#define ksi_five	KSI_MK_SINT (5)
#define ksi_six		KSI_MK_SINT (6)
#define ksi_seven	KSI_MK_SINT (7)
#define ksi_eight	KSI_MK_SINT (8)
#define ksi_nine	KSI_MK_SINT (9)
#define ksi_ten		KSI_MK_SINT (10)

#define ksi_eof		KSI_MK_TC8 (0, KSI_TC8_EOF)
#define ksi_nil		KSI_MK_TC8 (0, KSI_TC8_NIL)
#define ksi_err		KSI_MK_TC8 (0, KSI_TC8_ERR)
#define ksi_any		KSI_MK_TC8 (0, KSI_TC8_ANY)

#define errlog_emerg	KSI_MK_SINT (ERRLOG_EMERG)
#define errlog_alert	KSI_MK_SINT (ERRLOG_ALERT)
#define errlog_crit	KSI_MK_SINT (ERRLOG_CRIT)
#define errlog_error	KSI_MK_SINT (ERRLOG_ERROR)
#define errlog_warning	KSI_MK_SINT (ERRLOG_WARNING)
#define errlog_notice	KSI_MK_SINT (ERRLOG_NOTICE)
#define errlog_info	KSI_MK_SINT (ERRLOG_INFO)
#define errlog_debug	KSI_MK_SINT (ERRLOG_DEBUG)
#define errlog_all	KSI_MK_SINT (ERRLOG_ALL)


/* bool predicates */

#define KSI_EQ_P(x,y)		((x) == (y))
#define KSI_EQV_P(x,y)		(ksi_eqv_p ((x), (y)) != ksi_false)
#define KSI_EQUAL_P(x,y)	(ksi_equal_p ((x), (y)) != ksi_false)

#define KSI_TRUE_P(x)		((x) != ksi_false)
#define KSI_FALSE_P(x)		((x) == ksi_false)
#define KSI_VOID_P(x)		((x) == ksi_void)


/*
 * KSI_LIST1 ... KSI_LIST9	make list
 */

#define KSI_LIST1(x) \
    ksi_cons ((x), ksi_nil)

#define KSI_LIST2(x1,x2) \
    ksi_cons ((x1), KSI_LIST1 (x2))

#define KSI_LIST3(x1,x2,x3) \
    ksi_cons ((x1), KSI_LIST2 (x2,x3))

#define KSI_LIST4(x1,x2,x3,x4) \
    ksi_cons ((x1), KSI_LIST3 (x2,x3,x4))

#define KSI_LIST5(x1,x2,x3,x4,x5) \
    ksi_cons ((x1), KSI_LIST4 (x2,x3,x4,x5))

#define KSI_LIST6(x1,x2,x3,x4,x5,x6) \
    ksi_cons ((x1), KSI_LIST5 (x2,x3,x4,x5,x6))

#define KSI_LIST7(x1,x2,x3,x4,x5,x6,x7) \
    ksi_cons ((x1), KSI_LIST6 (x2,x3,x4,x5,x6,x7))

#define KSI_LIST8(x1,x2,x3,x4,x5,x6,x7,x8) \
    ksi_cons ((x1), KSI_LIST7 (x2,x3,x4,x5,x6,x7,x8))

#define KSI_LIST9(x1,x2,x3,x4,x5,x6,x7,x8,x9) \
    ksi_cons ((x1), KSI_LIST8 (x2,x3,x4,x5,x6,x7,x8,x9))

/*
 * KSI_CONS2 ... KSI_CONS9	make improper list
 */

#define KSI_CONS2(x1,x2) \
    ksi_cons ((x1), (x2))

#define KSI_CONS3(x1,x2,x3) \
    ksi_cons ((x1), KSI_CONS2 (x2,x3))

#define KSI_CONS4(x1,x2,x3,x4) \
    ksi_cons ((x1), KSI_CONS3 (x2,x3,x4))

#define KSI_CONS5(x1,x2,x3,x4,x5) \
    ksi_cons ((x1), KSI_CONS4 (x2,x3,x4,x5))

#define KSI_CONS6(x1,x2,x3,x4,x5,x6) \
    ksi_cons ((x1), KSI_CONS5 (x2,x3,x4,x5,x6))

#define KSI_CONS7(x1,x2,x3,x4,x5,x6,x7) \
    ksi_cons ((x1), KSI_CONS6 (x2,x3,x4,x5,x6,x7))

#define KSI_CONS8(x1,x2,x3,x4,x5,x6,x7,x8) \
    ksi_cons ((x1), KSI_CONS7 (x2,x3,x4,x5,x6,x7,x8))

#define KSI_CONS9(x1,x2,x3,x4,x5,x6,x7,x8,x9) \
    ksi_cons ((x1), KSI_CONS8 (x2,x3,x4,x5,x6,x7,x8,x9))


#define KSI_ERR(a,s)         (ksi_exn_error("misc",a,s))
#define KSI_EXN_ERR(t,a,s)   (ksi_exn_error(t,a,s))

#if !defined(CAUTIOUS) || CAUTIOUS >= 1
# define KSI_CHECK(a,c,s)       ((void)((c)||KSI_ERR(a,s)))
# define KSI_EXN_CHECK(t,a,c,s) ((void)((c)||KSI_EXN_ERR(t,a,s)))
# define KSI_TYPE_CHECK(a,c,s)  KSI_EXN_CHECK("type",a,c,s)
# define KSI_RANGE_CHECK(a,c,s) KSI_EXN_CHECK("range",a,c,s)
# define KSI_WARN(a,t,s)        ((void)((t)||ksi_warn("%s: %w",s,a)))
#else
# define KSI_CHECK(a,t,s)
# define KSI_EXN_CHECK(t,a,c,s)
# define KSI_TYPE_CHECK(a,c,s)
# define KSI_RANGE_CHECK(a,c,s)
# define KSI_WARN(a,t,s)
#endif

#if CAUTIOUS >= 2
# define KSI_CHECK2(a,c,s)       ((void)((c)||KSI_ERR(a,s)))
# define KSI_EXN_CHECK2(t,a,c,s) ((void)((c)||KSI_EXN_ERR(t,a,s)))
# define KSI_TYPE_CHECK2(a,c,s)  KSI_EXN_CHECK("type",a,c,s)
# define KSI_RANGE_CHECK2(a,c,s) KSI_EXN_CHECK("range",a,c,s)
# ifdef __GNUC__
#  define KSI_ASSERT(t)	((t) || ksi_error ("assert failed: %s [%s %d] (%s)", \
					   #t,				     \
					   __FILE__,			     \
					   __LINE__,			     \
					   __PRETTY_FUNCTION__))
# else
#  define KSI_ASSERT(t)	((t) || ksi_error ("assert failed: %s [%s %d]",	\
					   #t,				\
					   __FILE__,			\
					   __LINE__))
# endif
#else
# define KSI_CHECK2(a,t,s)
# define KSI_EXN_CHECK2(t,a,c,s)
# define KSI_TYPE_CHECK2(a,c,s)
# define KSI_RANGE_CHECK2(a,c,s)
# define KSI_ASSERT(t)
#endif


#ifdef __cplusplus
extern "C" {
#endif

SI_API
const char*
ksi_version (void);

SI_API
int
ksi_major_version (void);

SI_API
int
ksi_minor_version (void);

SI_API
int
ksi_patch_level ();

SI_API
double
ksi_cpu_time (void);

SI_API
double
ksi_eval_time (void);

SI_API
double
ksi_real_time (void);

SI_API
unsigned
ksi_hash_str (const char* str, int len, unsigned n);


/* GC & memory allocation functions */

SI_API
void*
ksi_malloc (size_t sz);

SI_API
void*
ksi_malloc_data (size_t sz);

SI_API
void*
ksi_malloc_eternal (size_t sz);

SI_API
void*
ksi_malloc_typed (size_t sz, size_t first_ptr_off, size_t last_ptr_off);

SI_API
void*
ksi_realloc (void* ptr, size_t sz);

SI_API
void
ksi_free (void* ptr);

SI_API
void
ksi_register_finalizer (void* obj, ksi_finalizer_function proc, void* data);

#define ksi_unregister_finalizer(obj) ksi_register_finalizer (obj, 0, 0)

SI_API
int
ksi_expand_heap (unsigned sz);

SI_API
void
ksi_set_max_heap (unsigned sz);

SI_API
unsigned
ksi_heap_size (void);

SI_API
unsigned
ksi_heap_free (void);

SI_API
unsigned
ksi_gcollections (void);

SI_API
int
ksi_gcollect (int full);

SI_API
void
ksi_enable_gc (void);

SI_API
void
ksi_disable_gc (void);

SI_API
void
ksi_gc_protect (ksi_obj x);

SI_API
void
ksi_gc_unprotect (ksi_obj x);


/* General system and file interface functions. */

SI_API
char*
ksi_expand_file_name (const char* fname);

SI_API
int
ksi_has_suffix (const char* fname, const char* suffix);

SI_API
void
ksi_system (const char* command);


/* Windows specific functions */

#ifdef MSWIN32

SI_API
int
ksi_win32s (void);

SI_API
int
ksi_winnt (void);

SI_API
int
ksi_win_console (void);

SI_API
const char*
ksi_win_name (void);

SI_API
void
ksi_init_console (char* console_title);

SI_API
void
ksi_register_dll (void* data_seg_ptr);

SI_API
const char*
ksi_get_last_error (char *msg);

SI_API
const char*
ksi_win_error_text (char *msg, unsigned long errcode);

SI_API
int
ksi_tolower (int x);

SI_API
int
ksi_toupper (int x);

SI_API
int
ksi_isalpha (int x);

SI_API
int
ksi_isalnum (int x);

SI_API
int
ksi_isupper (int x);

SI_API
int
ksi_islower (int x);

#endif


/* Initialization & termination. */

SI_API
ksi_obj
ksi_interactive (ksi_obj x);

SI_API
int
ksi_std_ports_inited (void);

SI_API
void
ksi_init_std_ports (int in_fd, int out_fd, int err_fd);

SI_API
void
ksi_init (const char* app, int ac, char** av, ksi_stack top_stack_addr);

SI_API
void
ksi_term (void);

SI_API
void
ksi_add_boot_initializer (ksi_init_function proc, void* data);

SI_API
void
ksi_add_initializer (ksi_init_function proc, void* data);

SI_API
void
ksi_add_library (const char *path, const char *file,
		 ksi_init_function proc, void *data);

SI_API
ksi_obj
ksi_add_exit_handler (ksi_obj proc);

SI_API
ksi_obj
ksi_add_error_handler (ksi_obj proc);

SI_API
ksi_obj
ksi_remove_error_handler (ksi_obj proc);

SI_API
const char*
ksi_init_path (void);

SI_API
const char*
ksi_site_path (void);

SI_API
const char*
ksi_libs_path (void);

SI_API
const char*
ksi_app_path (void);

SI_API
const char*
ksi_app (void);


/* Error handling */

SI_API
ksi_obj
ksi_open_errlog (ksi_obj priority, ksi_obj port_or_fname);

SI_API
ksi_obj
ksi_errlog (ksi_obj module, ksi_obj priority, const char *fmt, ...);

SI_API
ksi_obj
ksi_errlog_priority (ksi_obj module, ksi_obj priority);

SI_API
int
ksi_warn (const char *fmt, ...);

SI_API
int
ksi_error (const char *fmt, ...);

SI_API
int
ksi_crit_error (ksi_obj priority, const char* fmt, ...);

SI_API
int
ksi_exn_error (const char *type, ksi_obj errobj, const char *fmt, ...);

SI_API
ksi_obj
ksi_make_exn (const char *type, ksi_obj errobj, const char *msg, ksi_obj pri);

SI_API
ksi_obj
ksi_exn_p (ksi_obj x);

SI_API
ksi_obj
ksi_exn_type (ksi_obj x);

SI_API
ksi_obj
ksi_exn_message (ksi_obj x);

SI_API
ksi_obj
ksi_exn_value (ksi_obj x);

SI_API
ksi_obj
ksi_exn_priority (ksi_obj x);

SI_API
ksi_obj
ksi_throw_error (ksi_obj exn);

SI_API
ksi_obj
ksi_throw_fatal (ksi_obj exn);


/*       / . */

SI_API
const char*
ksi_obj2str (ksi_obj x);

SI_API
const char*
ksi_obj2name (ksi_obj x);

SI_API
ksi_obj
ksi_str02obj (const char* str);

SI_API
ksi_obj
ksi_str2obj (const char* str, int len);


/* symbol */

SI_API
ksi_obj
ksi_intern (const char* name, int len);

#define ksi_str2sym(name,len) ksi_intern (name, len)

#define ksi_str02sym(name) ksi_intern (name, strlen (name))

SI_API
ksi_obj
ksi_ptr02sym (const char* name);

SI_API
ksi_obj
ksi_find_symbol (const char* name, int len);

SI_API
const char*
ksi_symbol2str (ksi_obj sym);


/* boolean */

SI_API
ksi_obj
ksi_int2bool (int x);

SI_API
int
ksi_bool2int (ksi_obj x);


/* char */

SI_API
ksi_obj
ksi_str2char (const char* name, int len);

SI_API
const char*
ksi_char2str (ksi_obj x);


/* string */

SI_API
ksi_obj
ksi_str2string (const char* str, int len);

SI_API
ksi_obj
ksi_str02string (const char* str);

SI_API
ksi_obj
ksi_ptr02string (const char* str);

SI_API
ksi_obj
ksi_ptr2string (const char* str, int len);

SI_API
ksi_obj
ksi_make_string (int len, char fill_char);

SI_API
const char*
ksi_string2str (ksi_obj x);

SI_API
const char*
ksi_safe_string_ptr (ksi_obj str);


/* keyword */

SI_API
ksi_obj
ksi_str2key (const char* name, int len);

#define ksi_str02key(name) ksi_str2key (name, strlen (name))

SI_API
ksi_obj
ksi_ptr02key (const char* name);

SI_API
const char*
ksi_key2str (ksi_obj key);


/* number */

SI_API
ksi_obj
ksi_str02num (const char* str, int radix);

SI_API
ksi_obj
ksi_int2num (long v);

SI_API
ksi_obj
ksi_uint2num (unsigned long v);

#ifdef LONG_LONG
SI_API
ksi_obj
ksi_long2num (LONG_LONG v);

SI_API
ksi_obj
ksi_ulong2num (unsigned LONG_LONG v);

#else
# define ksi_long2num  ksi_int2num
# define ksi_ulong2num ksi_uint2num
#endif

SI_API
ksi_obj
ksi_double2num (double v);

SI_API
ksi_obj
ksi_make_bignum (int sign_and_size, unsigned *digits);

SI_API
ksi_obj
ksi_rectangular (double r, double i);

SI_API
ksi_obj
ksi_polar (double x, double a);

SI_API
ksi_obj
ksi_double2exact (double d);

SI_API
char*
ksi_num2str (ksi_obj x, int radix);

SI_API
double
ksi_num2double (ksi_obj x);

SI_API
long
ksi_num2int (ksi_obj x);

SI_API
unsigned long
ksi_num2uint (ksi_obj x);

#ifdef LONG_LONG
SI_API
LONG_LONG
ksi_num2long (ksi_obj x);

SI_API
unsigned LONG_LONG
ksi_num2ulong (ksi_obj x);

#else
# define ksi_num2long  ksi_num2int
# define ksi_num2ulong ksi_num2uint
#endif

SI_API
double
ksi_real_part (ksi_obj x);

SI_API
double
ksi_imag_part (ksi_obj x);

SI_API
double
ksi_magnitude (ksi_obj x);

SI_API
double
ksi_angle (ksi_obj z);


/* list */

SI_API
int
ksi_list_len (ksi_obj x);


/* C-pointer */

SI_API
ksi_obj
ksi_make_pointer (void *val, size_t size);

SI_API
ksi_obj
ksi_make_const_pointer (const void *val, size_t size);


/*      */

/* Symbol utils */

SI_API
ksi_obj
ksi_symbol_p (ksi_obj x);

SI_API
ksi_obj
ksi_symbol2string (ksi_obj sym);

SI_API
ksi_obj
ksi_string2symbol (ksi_obj str);

SI_API
ksi_obj
ksi_gensym (ksi_obj name, ksi_obj unused);


/* boolean utils */

SI_API
ksi_obj
ksi_bool_p (ksi_obj x);

SI_API
ksi_obj
ksi_not (ksi_obj x);

SI_API
ksi_obj
ksi_eq_p (ksi_obj x1, ksi_obj x2);

SI_API
ksi_obj
ksi_eqv_p (ksi_obj x1, ksi_obj x2);

SI_API
ksi_obj
ksi_equal_p (ksi_obj x1, ksi_obj x2);

SI_API
ksi_obj
ksi_void_p (ksi_obj x);


/* Char utils */

SI_API
ksi_obj
ksi_char_p (ksi_obj x);

SI_API
ksi_obj
ksi_char_eq_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_char_lt_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_char_gt_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_char_le_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_char_ge_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_char_ci_eq_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_char_ci_lt_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_char_ci_gt_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_char_ci_le_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_char_ci_ge_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_char_alpha_p (ksi_obj x);

SI_API
ksi_obj
ksi_char_digit_p (ksi_obj x);

SI_API
ksi_obj
ksi_char_space_p (ksi_obj x);

SI_API
ksi_obj
ksi_char_upper_p (ksi_obj x);

SI_API
ksi_obj
ksi_char_lower_p (ksi_obj x);

SI_API
ksi_obj
ksi_char_upcase (ksi_obj x);

SI_API
ksi_obj
ksi_char_downcase (ksi_obj x);

SI_API
ksi_obj
ksi_char2integer (ksi_obj x);

SI_API
ksi_obj
ksi_integer2char (ksi_obj x);


/* String utils */

SI_API
ksi_obj
ksi_string_p (ksi_obj x);

SI_API
ksi_obj
ksi_list2string (ksi_obj list);

SI_API
ksi_obj
ksi_string_length (ksi_obj str);

SI_API
ksi_obj
ksi_string_ref (ksi_obj str, ksi_obj k);

SI_API
ksi_obj
ksi_string_set_x (ksi_obj str, ksi_obj k, ksi_obj c);

SI_API
ksi_obj
ksi_substring (ksi_obj str, ksi_obj start, ksi_obj end);

SI_API
ksi_obj
ksi_string_append (int argc, ksi_obj* args);

SI_API
ksi_obj
ksi_string_copy (ksi_obj str);

SI_API
ksi_obj
ksi_string_fill_x (ksi_obj str, ksi_obj fill_char);

SI_API
ksi_obj
ksi_string_eqv_p (ksi_obj s1, ksi_obj s2);

SI_API
ksi_obj
ksi_string_equal_p (ksi_obj s1, ksi_obj s2);

SI_API
ksi_obj
ksi_string_ci_equal_p (ksi_obj s1, ksi_obj s2);

SI_API
ksi_obj
ksi_string_eq_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_string_ci_eq_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_string_ls_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_string_gt_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_string_le_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_string_ge_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_string_ci_ls_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_string_ci_gt_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_string_ci_le_p (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_string_ci_ge_p (int argc, ksi_obj* argv);


/* Keyword utils */

SI_API
ksi_obj
ksi_key_p (ksi_obj x);

SI_API
ksi_obj
ksi_string2keyword (ksi_obj str);

SI_API
ksi_obj
ksi_keyword2string (ksi_obj key);

SI_API
ksi_obj
ksi_symbol2keyword (ksi_obj sym);

SI_API
ksi_obj
ksi_keyword2symbol (ksi_obj key);

SI_API
ksi_obj
ksi_make_keyword (ksi_obj x);

SI_API
ksi_obj
ksi_get_keyword (ksi_obj key, ksi_obj list, ksi_obj def);


/* Number utils */

SI_API
ksi_obj
ksi_number_p (ksi_obj x);

SI_API
ksi_obj
ksi_complex_p (ksi_obj x);

SI_API
ksi_obj
ksi_real_p (ksi_obj x);

SI_API
ksi_obj
ksi_rational_p (ksi_obj x);

SI_API
ksi_obj
ksi_integer_p (ksi_obj x);

SI_API
ksi_obj
ksi_exact_p (ksi_obj x);

SI_API
ksi_obj
ksi_inexact_p (ksi_obj x);

SI_API
ksi_obj
ksi_number2string (ksi_obj num, ksi_obj rad);

SI_API
ksi_obj
ksi_string2number (ksi_obj str, ksi_obj rad);

SI_API
ksi_obj
ksi_exact2inexact (ksi_obj x);

SI_API
ksi_obj
ksi_inexact2exact (ksi_obj x);

SI_API
ksi_obj
ksi_abs (ksi_obj x);

SI_API
ksi_obj
ksi_min (int argc, ksi_obj* args);

SI_API
ksi_obj
ksi_max (int argc, ksi_obj* args);

SI_API
ksi_obj
ksi_add (ksi_obj a, ksi_obj b);

SI_API
ksi_obj
ksi_sub (ksi_obj a, ksi_obj b);

SI_API
ksi_obj
ksi_mul (ksi_obj a, ksi_obj b);

SI_API
ksi_obj
ksi_div (ksi_obj a, ksi_obj b);

SI_API
ksi_obj
ksi_quot (ksi_obj a, ksi_obj b);

SI_API
ksi_obj
ksi_rem (ksi_obj a, ksi_obj b);

SI_API
ksi_obj
ksi_mod (ksi_obj a, ksi_obj b);

SI_API
ksi_obj
ksi_gcd (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_lcm (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_floor (ksi_obj x);

SI_API
ksi_obj
ksi_ceiling (ksi_obj x);

SI_API
ksi_obj
ksi_truncate (ksi_obj x);

SI_API
ksi_obj
ksi_round (ksi_obj x);

SI_API
ksi_obj
ksi_sqrt (ksi_obj x);

SI_API
ksi_obj
ksi_expt (ksi_obj z1, ksi_obj z2);

SI_API
ksi_obj
ksi_exp (ksi_obj x);

SI_API
ksi_obj
ksi_log (ksi_obj x);

SI_API
ksi_obj
ksi_sin (ksi_obj x);

SI_API
ksi_obj
ksi_cos (ksi_obj x);

SI_API
ksi_obj
ksi_tan (ksi_obj x);

SI_API
ksi_obj
ksi_asin (ksi_obj x);

SI_API
ksi_obj
ksi_acos (ksi_obj x);

SI_API
ksi_obj
ksi_atan (ksi_obj y, ksi_obj x);

SI_API
ksi_obj
ksi_sinh (ksi_obj x);

SI_API
ksi_obj
ksi_cosh (ksi_obj x);

SI_API
ksi_obj
ksi_tanh (ksi_obj x);

SI_API
ksi_obj
ksi_asinh (ksi_obj x);

SI_API
ksi_obj
ksi_acosh (ksi_obj x);

SI_API
ksi_obj
ksi_atanh (ksi_obj y);

SI_API
ksi_obj
ksi_zero_p (ksi_obj x);

SI_API
ksi_obj
ksi_positive_p (ksi_obj x);

SI_API
ksi_obj
ksi_negative_p (ksi_obj x);

SI_API
ksi_obj
ksi_odd_p (ksi_obj x);

SI_API
ksi_obj
ksi_even_p (ksi_obj x);

SI_API
ksi_obj
ksi_num_eqv_p (ksi_obj x1, ksi_obj x2);

#define ksi_num_equal_p(x1, x2) ksi_num_eqv_p (x1, x2)

SI_API
ksi_obj
ksi_num_eq_p (int argc, ksi_obj* args);

SI_API
ksi_obj
ksi_num_lt_p (int argc, ksi_obj* args);

SI_API
ksi_obj
ksi_num_le_p (int argc, ksi_obj* args);

SI_API
ksi_obj
ksi_num_gt_p (int argc, ksi_obj* args);

SI_API
ksi_obj
ksi_num_ge_p (int argc, ksi_obj* args);

SI_API
ksi_obj
ksi_lognot (ksi_obj x);

SI_API
ksi_obj
ksi_logior (ksi_obj x, ksi_obj y);

SI_API
ksi_obj
ksi_logxor (ksi_obj x, ksi_obj y);

SI_API
ksi_obj
ksi_logand (ksi_obj x, ksi_obj y);

SI_API
ksi_obj
ksi_logtest (ksi_obj x, ksi_obj y);

SI_API
ksi_obj
ksi_logbit_p (ksi_obj ind, ksi_obj x);

SI_API
ksi_obj
ksi_ash (ksi_obj x, ksi_obj cnt);


/* Pair & List utilites */

SI_API
ksi_obj
ksi_pair_p (ksi_obj x);

SI_API
ksi_obj
ksi_null_p (ksi_obj x);

SI_API
ksi_obj
ksi_list_p (ksi_obj x);

SI_API
ksi_obj
ksi_cons (ksi_obj car, ksi_obj cdr);

SI_API
ksi_obj
ksi_acons (ksi_obj car, ksi_obj cdr, ksi_obj alist);

SI_API
ksi_obj
ksi_car (ksi_obj);

SI_API
ksi_obj
ksi_cdr (ksi_obj);

SI_API
ksi_obj
ksi_set_car_x (ksi_obj pair, ksi_obj val);

SI_API
ksi_obj
ksi_set_cdr_x (ksi_obj pair, ksi_obj val);

SI_API
ksi_obj
ksi_length (ksi_obj list);

SI_API
ksi_obj
ksi_make_list (ksi_obj len, ksi_obj init);

SI_API
ksi_obj
ksi_append (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_append_x (int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_reverse (ksi_obj list);

SI_API
ksi_obj
ksi_reverse_x (ksi_obj arg);

SI_API
ksi_obj
ksi_list_head (ksi_obj list, ksi_obj len);

SI_API
ksi_obj
ksi_list_tail (ksi_obj list, ksi_obj len);

SI_API
ksi_obj
ksi_list_ref (ksi_obj list, ksi_obj num);

SI_API
ksi_obj
ksi_list_set_x (ksi_obj lst, ksi_obj num, ksi_obj val);

SI_API
ksi_obj
ksi_last_pair (ksi_obj x);

SI_API
ksi_obj
ksi_copy_list (ksi_obj o);

SI_API
ksi_obj
ksi_copy_tree (ksi_obj o);

SI_API
ksi_obj
ksi_memq (ksi_obj o, ksi_obj list);

SI_API
ksi_obj
ksi_memv (ksi_obj o, ksi_obj list);

SI_API
ksi_obj
ksi_member (ksi_obj o, ksi_obj list, ksi_obj proc);

SI_API
ksi_obj
ksi_memq_remove (ksi_obj x, ksi_obj lst);

SI_API
ksi_obj
ksi_memv_remove (ksi_obj x, ksi_obj lst);

SI_API
ksi_obj
ksi_member_remove (ksi_obj x, ksi_obj lst, ksi_obj proc);

SI_API
ksi_obj
ksi_memq_remove_x (ksi_obj x, ksi_obj lst);

SI_API
ksi_obj
ksi_memv_remove_x (ksi_obj x, ksi_obj lst);

SI_API
ksi_obj
ksi_member_remove_x (ksi_obj x, ksi_obj lst, ksi_obj proc);

SI_API
ksi_obj
ksi_assq (ksi_obj o, ksi_obj list);

SI_API
ksi_obj
ksi_assv (ksi_obj o, ksi_obj list);

SI_API
ksi_obj
ksi_assoc (ksi_obj o, ksi_obj list, ksi_obj proc);

SI_API
ksi_obj
ksi_assq_ref (ksi_obj alist, ksi_obj key);

SI_API
ksi_obj
ksi_assv_ref (ksi_obj alist, ksi_obj key);

SI_API
ksi_obj
ksi_assoc_ref (ksi_obj alist, ksi_obj key, ksi_obj proc);

SI_API
ksi_obj
ksi_assq_set_x (ksi_obj alist, ksi_obj key, ksi_obj val);

SI_API
ksi_obj
ksi_assv_set_x (ksi_obj alist, ksi_obj key, ksi_obj val);

SI_API
ksi_obj
ksi_assoc_set_x (ksi_obj alist, ksi_obj key, ksi_obj val, ksi_obj proc);

SI_API
ksi_obj
ksi_assq_remove_x (ksi_obj alist, ksi_obj key);

SI_API
ksi_obj
ksi_assv_remove_x (ksi_obj alist, ksi_obj key);

SI_API
ksi_obj
ksi_assoc_remove_x (ksi_obj alist, ksi_obj key, ksi_obj proc);


/* Vector utils */

SI_API
ksi_obj
ksi_vector_p (ksi_obj x);

SI_API
ksi_obj
ksi_copy_vector (ksi_obj x);

SI_API
ksi_obj
ksi_list2vector (ksi_obj x);

SI_API
ksi_obj
ksi_vector2list (ksi_obj v);

SI_API
ksi_obj
ksi_make_vector (ksi_obj k, ksi_obj fill);

SI_API
ksi_obj
ksi_vector_length (ksi_obj v);

SI_API
ksi_obj
ksi_vector_ref (ksi_obj vec, ksi_obj k);

SI_API
ksi_obj
ksi_vector_set_x (ksi_obj vec, ksi_obj k, ksi_obj val);

SI_API
ksi_obj
ksi_vector_fill_x (ksi_obj vec, ksi_obj fill);

SI_API
ksi_obj
ksi_vector_map (ksi_obj proc, ksi_obj vec);

SI_API
ksi_obj
ksi_vector_for_each (ksi_obj proc, ksi_obj vec);


/* Port utils */

SI_API
ksi_obj
ksi_port_p (ksi_obj x);

SI_API
ksi_obj
ksi_input_port_p (ksi_obj x);

SI_API
ksi_obj
ksi_output_port_p (ksi_obj x);

SI_API
ksi_obj
ksi_null_port (void);

SI_API
ksi_obj
ksi_current_input_port (void);

SI_API
ksi_obj
ksi_current_output_port (void);

SI_API
ksi_obj
ksi_current_error_port (void);

SI_API
ksi_obj
ksi_set_current_output_port (ksi_obj port);

SI_API
ksi_obj
ksi_set_current_input_port (ksi_obj port);

SI_API
ksi_obj
ksi_set_current_error_port (ksi_obj port);

SI_API
ksi_obj
ksi_open_file (ksi_obj fname, ksi_obj mode);

SI_API
ksi_obj
ksi_open_string (ksi_obj str, ksi_obj mode);

SI_API
ksi_obj
ksi_port_string (ksi_obj str_port);

SI_API
ksi_obj
ksi_close_port (ksi_obj port);

SI_API
ksi_obj
ksi_flush_port (ksi_obj port);


/* Input/output */

SI_API
ksi_obj
ksi_printf (ksi_obj port, const char* fmt, ...);

SI_API
ksi_obj
ksi_vprintf (ksi_obj port, const char* fmt, va_list args);

SI_API
char*
ksi_aprintf (const char* fmt, ...);

SI_API
ksi_obj
ksi_format (ksi_obj port, const char* fmt, int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_object2string (ksi_obj x);

SI_API
ksi_obj
ksi_read_char (ksi_obj p);

SI_API
ksi_obj
ksi_peek_char (ksi_obj p);

SI_API
ksi_obj
ksi_char_ready_p (ksi_obj p);

SI_API
ksi_obj
ksi_port_ready_p (ksi_obj p);

SI_API
ksi_obj
ksi_write_char (ksi_obj o, ksi_obj p);

SI_API
ksi_obj
ksi_eof_object_p (ksi_obj x);

SI_API
ksi_obj
ksi_read (ksi_obj port);

SI_API
ksi_obj
ksi_read_const (ksi_obj port);

SI_API
ksi_obj
ksi_newline (ksi_obj port);

SI_API
ksi_obj
ksi_write (ksi_obj o, ksi_obj port);

SI_API
ksi_obj
ksi_display (ksi_obj o, ksi_obj port);

SI_API
ksi_obj
ksi_read_block (ksi_obj port, ksi_obj size);

SI_API
ksi_obj
ksi_write_block (ksi_obj port, ksi_obj str);

SI_API
ksi_obj
ksi_set_async_mode (ksi_obj x, ksi_obj async);


/* dump & undump ksi objects */

SI_API
void*
ksi_dump (ksi_obj x, int* size);

SI_API
ksi_obj
ksi_undump (void* data, int size);

SI_API
ksi_obj
ksi_obj2dump (ksi_obj x);

SI_API
ksi_obj
ksi_dump2obj (ksi_obj x);

SI_API
ksi_obj
ksi_read_dump (ksi_obj port);

SI_API
ksi_obj
ksi_write_dump (ksi_obj x, ksi_obj port);


/* Control & eval utils */

SI_API
ksi_obj
ksi_procedure_p (ksi_obj x);

SI_API
ksi_obj
ksi_procedure_arity (ksi_obj proc);

SI_API
ksi_obj
ksi_procedure_has_arity_p (ksi_obj proc, ksi_obj min_args, ksi_obj max_args);

SI_API
ksi_obj
ksi_eval (ksi_obj form, ksi_obj env);

SI_API
ksi_obj
ksi_eval_with_catch (ksi_obj form, ksi_obj env);

SI_API
ksi_obj
ksi_eval_str (const char* scheme_code);

SI_API
ksi_obj
ksi_eval_str_with_catch (const char* scheme_code);

SI_API
ksi_obj
ksi_apply_proc (ksi_obj proc, int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_apply_proc_with_catch (ksi_obj proc, int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_apply_0 (ksi_obj proc);

SI_API
ksi_obj
ksi_apply_1 (ksi_obj proc, ksi_obj arg1);

SI_API
ksi_obj
ksi_apply_2 (ksi_obj proc, ksi_obj arg1, ksi_obj arg2);

SI_API
ksi_obj
ksi_apply_3 (ksi_obj proc, ksi_obj arg1, ksi_obj arg2, ksi_obj arg3);

SI_API
ksi_obj
ksi_apply (ksi_obj proc, ksi_obj arg_list);

SI_API
ksi_obj
ksi_apply_0_with_catch (ksi_obj proc);

SI_API
ksi_obj
ksi_apply_1_with_catch (ksi_obj proc, ksi_obj arg1);

SI_API
ksi_obj
ksi_apply_2_with_catch (ksi_obj proc, ksi_obj arg1, ksi_obj arg2);

SI_API
ksi_obj
ksi_apply_3_with_catch (ksi_obj proc, ksi_obj arg1, ksi_obj arg2, ksi_obj arg3);

SI_API
ksi_obj
ksi_apply_with_catch (ksi_obj proc, ksi_obj arg_list);

SI_API
ksi_obj
ksi_map (ksi_obj proc, int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_for_each (ksi_obj proc, int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_fold_left (ksi_obj kons, ksi_obj knil, int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_fold_right (ksi_obj kons, ksi_obj knil, int argc, ksi_obj* argv);

SI_API
ksi_obj
ksi_force (ksi_obj x);

SI_API
ksi_obj
ksi_call_with_values (ksi_obj thunk, ksi_obj proc);

SI_API
ksi_obj
ksi_dynamic_wind (ksi_obj pre, ksi_obj act, ksi_obj post);

SI_API
ksi_obj
ksi_call_cc (ksi_obj proc);

SI_API
ksi_obj
ksi_catch (ksi_obj tag, ksi_obj body, ksi_obj handler);

SI_API
ksi_obj
ksi_catch_with_retry (ksi_obj tag, ksi_obj body, ksi_obj handler);

SI_API
ksi_obj
ksi_throw (ksi_obj tag, ksi_obj val);

SI_API
ksi_obj
ksi_exit (ksi_obj x);

SI_API
ksi_obj
ksi_quit (void);

SI_API
int
ksi_call_with_dynamic_context (int (*proc) (void *data),
			       int (*handler) (void *data),
			       void *data);


/* Environ utils */

SI_API
ksi_obj
ksi_defsym (const char* name, ksi_obj val, ksi_obj env);

SI_API
ksi_obj
ksi_spec_val (const char* name, ksi_obj def, ksi_obj env);

SI_API
ksi_obj
ksi_glob_val (ksi_obj sym, ksi_obj def);

SI_API
ksi_obj
ksi_env_p (ksi_obj x);

SI_API
ksi_obj
ksi_global_env (void);

SI_API
ksi_obj
ksi_current_env (void);

SI_API
ksi_obj
ksi_make_env (ksi_obj src_env);

SI_API
ksi_obj
ksi_define (ksi_obj sym, ksi_obj val, ksi_obj env);

SI_API
ksi_obj
ksi_defined_p (ksi_obj sym, ksi_obj env);

SI_API
ksi_obj
ksi_var_p (ksi_obj env, ksi_obj sym);

SI_API
ksi_obj
ksi_var_ref (ksi_obj env, ksi_obj sym);

SI_API
ksi_obj
ksi_var_set (ksi_obj env, ksi_obj sym, ksi_obj val);


/* hashing */

SI_API
ksi_obj
ksi_hash (ksi_obj x, ksi_obj n);

SI_API
ksi_obj
ksi_hashv (ksi_obj x, ksi_obj n);

SI_API
ksi_obj
ksi_hashq (ksi_obj x, ksi_obj n);


/* hash-tables */

SI_API
ksi_obj
ksi_hashtab_p (ksi_obj x);

SI_API
ksi_obj
ksi_make_hashtab (ksi_obj size, ksi_obj hash, ksi_obj cmp);

SI_API
ksi_obj
ksi_hash_ref (ksi_obj tab, ksi_obj key, ksi_obj def);

SI_API
ksi_obj
ksi_hash_set_x (ksi_obj tab, ksi_obj key, ksi_obj val);

SI_API
ksi_obj
ksi_hash_remove_x (ksi_obj tab, ksi_obj key);

SI_API
ksi_obj
ksi_hash_map (ksi_obj proc, ksi_obj tab);

SI_API
ksi_obj
ksi_hash_for_each (ksi_obj proc, ksi_obj tab);


/* Loading */

SI_API
ksi_obj
ksi_load (ksi_obj str);

SI_API
ksi_obj
ksi_try_load (ksi_obj str, ksi_obj init);

SI_API
void
ksi_load_init_file (const char* fn);


/* Dynamic loading */

SI_API
ksi_obj
ksi_dynamic_link (ksi_obj mod, ksi_obj sym);

SI_API
ksi_obj
ksi_dynamic_call (ksi_obj func, ksi_obj arg_list);

SI_API
ksi_obj
ksi_dynamic_unlink (ksi_obj func);


/* Klos */

SI_API
ksi_obj
ksi_instance_p (ksi_obj x);

SI_API
ksi_obj
ksi_class_p (ksi_obj x);

SI_API
ksi_obj
ksi_generic_p (ksi_obj x);

SI_API
ksi_obj
ksi_method_p (ksi_obj x);

SI_API
ksi_obj
ksi_slot_ref (ksi_obj obj, ksi_obj name);

SI_API
ksi_obj
ksi_slot_set (ksi_obj obj, ksi_obj name, ksi_obj val);

SI_API
ksi_obj
ksi_slot_bound_p (ksi_obj obj, ksi_obj name);

SI_API
ksi_obj
ksi_slot_exist_p (ksi_obj obj, ksi_obj name);

SI_API
ksi_obj
ksi_slot_exist_in_class_p (ksi_obj cls, ksi_obj name);

SI_API
ksi_obj
ksi_class_of (ksi_obj x);

SI_API
ksi_obj
ksi_type_p (ksi_obj type, ksi_obj x);

SI_API
ksi_obj
ksi_type_of (ksi_obj x);

SI_API
ksi_obj
ksi_get_arg (ksi_obj key, ksi_obj args, ksi_obj def);


/* events */

SI_API
ksi_obj
ksi_event_p (ksi_obj x);

SI_API
ksi_obj
ksi_event_state (ksi_obj x);

SI_API
ksi_obj
ksi_event_procedure (ksi_obj x);

SI_API
ksi_obj
ksi_event_result (ksi_obj x);

SI_API
ksi_obj
ksi_start_event (ksi_obj x);

SI_API
ksi_obj
ksi_stop_event (ksi_obj x);

SI_API
ksi_obj
ksi_wait_event (ksi_obj tm);

SI_API
ksi_obj
ksi_sleep (ksi_obj tm);

SI_API
ksi_obj
ksi_enable_evt (void);

SI_API
ksi_obj
ksi_disable_evt (void);

SI_API
ksi_obj
ksi_ready_event (ksi_obj proc);

SI_API
ksi_obj
ksi_timer_event (ksi_obj tm, ksi_obj proc);

SI_API
ksi_obj
ksi_input_event (ksi_obj tm, ksi_obj port, ksi_obj proc);

SI_API
ksi_obj
ksi_output_event (ksi_obj tm, ksi_obj port, ksi_obj proc);

SI_API
ksi_obj
ksi_signal_event (ksi_obj signum, ksi_obj proc);

SI_API
ksi_obj
ksi_idle_event (ksi_obj proc);


#ifdef __cplusplus
}
#endif


#endif

/* End of code */
