/* -*-mode:C++-*- */
/*
 * ksi_conf.h
 * ksi configuration
 *
 * Copyright (C) 1997-1999,2000,2001,2002, ivan demakov
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without a written agreement
 * is hereby granted, provided that the above copyright notice and this
 * paragraph and the following two paragraphs appear in all copies.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE AUTHORS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *
 * Author:        Ivan Demakov <demakov@users.sourceforge.net>
 * Creation date: Thu Jul 31 19:27:23 1997
 * Last Update:   Fri Feb  8 18:24:04 2002
 *
 *
 * $Id: ksi_conf.h,v 1.13 2002/02/08 12:51:37 ksion Exp $
 *
 */

#ifndef KSI_CONF_H
#define KSI_CONF_H

#include <ksi_ver.h>
#include <ksi_path.h>


#if !defined(unix) && (defined(__unix) || defined(__unix__))
#  define unix 1
#endif

#ifdef HAVE_CONFIG_H
#  include <config.h>
#  if SIZEOF_LONG_LONG > 0
#    define HAVE_LONG_LONGS 1
#    define LONG_LONG long long
#  endif
#elif defined(__WATCOMC__)
#  define STDC_HEADERS		1
#  define HAVE_CTYPE_H		1
#  define HAVE_LIMITS_H		1
#  define HAVE_STDARG_H		1
#  define HAVE_STDDEF_H		1
#  define HAVE_STDLIB_H		1
#  define HAVE_STDIO_H		1
#  define HAVE_STRING_H		1
#  define HAVE_MALLOC_H		1
#  define HAVE_SYS_TYPES_H	1
#  define HAVE_TIME_H		1
#  define HAVE_SYS_TIMEB_H	1
#  define HAVE_SYS_STAT_H	1
#  define HAVE_FCNTL_H		1
#  define HAVE_UNISTD_H		1
#  define HAVE_ERRNO_H		1
#  define HAVE_MEMSET		1
#  define HAVE_MEMCPY		1
#  define HAVE_MEMMOVE		1
#  define HAVE_FTIME		1
#  define HAVE_RENAME		1
#  define SIZEOF_SHORT		2
#  define SIZEOF_INT		4
#  define SIZEOF_LONG		4
#  define SIZEOF_VOIDP		4
#  ifndef __cplusplus
#    define inline
#  endif
#elif defined(_MSC_VER)
#  define STDC_HEADERS        1
#  define HAVE_CTYPE_H        1
#  define HAVE_LIMITS_H       1
#  define HAVE_STDARG_H       1
#  define HAVE_STDDEF_H       1
#  define HAVE_STDLIB_H       1
#  define HAVE_STDIO_H        1
#  define HAVE_STRING_H       1
#  define HAVE_MALLOC_H       1
#  define HAVE_SYS_TYPES_H    1
#  define HAVE_TIME_H         1
#  define HAVE_SYS_TIMEB_H    1
#  define HAVE_SYS_STAT_H     1
#  define HAVE_FCNTL_H        1
#  define HAVE_ERRNO_H        1
#  define HAVE_MEMSET	      1
#  define HAVE_MEMCPY         1
#  define HAVE_MEMMOVE        1
#  define HAVE_FTIME          1
#  define HAVE_RENAME         1
#  define SIZEOF_SHORT        2
#  define SIZEOF_INT          4
#  define SIZEOF_LONG         4
#  define SIZEOF_VOIDP        4
#  define HAVE_LONG_LONGS     1
#  define LONG_LONG           __int64
#  define SIZEOF_LONG_LONG    sizeof(LONG_LONG)
#  ifndef __cplusplus
#    define inline __inline
#  endif
#endif

/* AIX requires this to be the first thing in the file.  */
#ifndef __GNUC__
# if HAVE_ALLOCA_H
#  include <alloca.h>
# else
#  ifdef _AIX
#pragma alloca
#  else
#   ifndef alloca /* predefined by HP cc +Olibcalls */
      char *alloca ();
#   endif
#  endif
# endif
#endif


#ifdef HAVE_CTYPE_H
#  include <ctype.h>
#endif
#ifdef HAVE_LIMITS_H
#  include <limits.h>
#endif
#ifdef HAVE_MEMORY_H
#  include <memory.h>
#endif
#ifdef HAVE_STDARG_H
#  include <stdarg.h>
#endif
#ifdef HAVE_STDDEF_H
#  include <stddef.h>
#endif
#ifdef HAVE_STDLIB_H
#  include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#  include <string.h>
#endif
#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif
#ifdef HAVE_MALLOC_H
#  include <malloc.h>
#endif
#ifdef HAVE_ERRNO_H
#  include <errno.h>
#elif defined(HAVE_SYS_ERRNO_H)
#  include <sys/errno.h>
#else
#  ifdef __cplusplus
     extern "C" int errno;
#  else
     extern int errno;
#  endif
#endif

#ifdef MSWIN32
#  undef tolower
#  define tolower ksi_tolower

#  undef toupper
#  define toupper ksi_toupper

#  undef isalpha
#  define isalpha ksi_isalpha

#  undef isalnum
#  define isalnum ksi_isalnum

#  undef isupper
#  define isupper ksi_isupper

#  undef islower
#  define islower ksi_islower
#endif

#ifndef HAVE_BZERO
#  ifdef HAVE_MEMSET
#    define bzero(p,n) memset(p,0,(size_t)(n))
#  else
     exterm void bzero(void* p, int n);
#  endif
#endif

#ifndef HAVE_MEMMOVE
#  ifdef HAVE_BCOPY
#    define memmove(dst,src,len) bcopy(src,dst,len)
#  else
     extern void memmove(void*, const void*, size_t);
#  endif
#endif

#ifndef HAVE_STRCASECMP
#  if defined(__WATCOMC__)
#    define strcasecmp stricmp
#  elif defined(_MSC_VER)
#    define strcasecmp _stricmp
#  endif
#endif

#if defined(__WATCOMC__) || defined(_MSC_VER)
#  define ABS(x)		(abs ((x)))
#else
#  ifndef ABS
#    define ABS(x)		((x) < 0 ? -(x) : (x))
#  endif
#endif

#if defined(__WATCOMC__)
#  define _get_osfhandle  _os_handle
#  define _open_osfhandle _hdopen
#endif

#if defined(__GNUC__) && defined(__cplusplus)
#  define MAX(a,b)            ((a) >? (b))
#  define MIN(a,b)            ((a) <? (b))
#else
#  define MAX(a,b)            ((a) > (b) ? (a) : (b))
#  define MIN(a,b)            ((a) < (b) ? (a) : (b))
#endif

#ifndef offsetof
#  define offsetof(t,m) ((size_t) & (((t*) 0) -> m))
#endif

#if defined(MSWIN32) || defined(MSDOS) || defined(OS2) || defined(__CYGWIN32__) || defined(__CYGWIN__)
#  define strpathcmp strcasecmp
#else
#  define strpathcmp strcmp
#endif


/*
 * CAUTIOUS: 0 - no checks, fastest but dangerous
 *           1 - normal checks, default
 *           2 - paranoid checks, more error messages
 *           3 - debug the interpreter
 */

#ifndef CAUTIOUS
# define CAUTIOUS 1
#endif


/* STRICT_R5RS: 0 - relax syntax, default
 *              1 - aditional syntax checks for strong R5RS compliance
 *
 */

#ifndef STRICT_R5RS
# define STRICT_R5RS 0
#endif


/*
 * GC_INCREMENTAL: if not zero, enable incremental garbage collection.
 * Not recomended, if GC library is not supports incremental
 * collection directly (see GC documentation).
 */

#ifndef GC_INCREMENTAL
# define GC_INCREMENTAL 0
#endif

#ifndef KSI_HOST
# if defined(unix)
#  define KSI_HOST	"unix"
# elif defined(__CYGWIN32__) || defined(__CYGWIN__)
#  define KSI_HOST	"cygwin"
# elif defined(MSWIN32)
#  define KSI_HOST	"win32"
# elif defined(OS2)
#  define KSI_HOST	"os/2"
# elif defined(MSDOS)
#  define KSI_HOST	"dos"
# else
#  define KSI_HOST	"unknown"
# endif
#endif

/*
 * KSI_ENV_LIB
 *	Environ variable name where ksi library path stored
 */

#ifndef KSI_ENV_LIB
# define KSI_ENV_LIB "KSI_LIBRARY"
#endif

/*
 * KSI_LIB_DIR
 *	Default ksi library path. Used if KSI_ENV_LIB not set.
 *	Normaly defined in config.h by configure script.
 */

#ifndef KSI_LIB_DIR
# if defined(unix)
#  define KSI_LIB_DIR "/usr/local/lib/ksi"
# elif defined(__CYGWIN32__) || defined(__CYGWIN__)
#  define KSI_LIB_DIR "/ksi"
# elif defined(MSDOS) || defined(MSWIN32) || defined(OS2)
#  define KSI_LIB_DIR "c:\\ksi\\lib"
# else
#  define KSI_LIB_DIR ""
# endif
#endif

/*
 * SITE_DIR	subdirectory for site specific files
 * APP_DIR	subdirectory for application specific files
 */

#ifndef SITE_DIR
# define SITE_DIR "site"
#endif

#ifndef APP_DIR
# define APP_DIR "app"
#endif


/*
 * KSI_SCM_SUFFIX	suffux for files with Scheme code
 * KSI_KO_SUFFIX	suffix for compiled code
 */

#ifndef KSI_SCM_SUFFIX
# define KSI_SCM_SUFFIX ".scm"
#endif

#ifndef KSI_KO_SUFFIX
# define KSI_KO_SUFFIX ".ko"
#endif


/*
 * KSI_DL_SUFFIX
 *	default suffix for dynamic libraries
 */

#ifndef KSI_DL_SUFFIX
#  if defined(HAVE_LIBDL) || defined(HAVE_DLOPEN)
#    define KSI_DL_SUFFIX	".so"
#  elif defined(HAVE_LIBDLD)
#    define KSI_DL_SUFFIX	".so" /* FIXME: what the suffix ? */
#  elif defined(HAVE_SHL_LOAD)
#    define KSI_DL_SUFFIX	".shl" /* FIXME: what the suffix ? */
#  elif defined(MSWIN32) || defined(OS2) || defined(MSDOS)
#    define KSI_DL_SUFFIX	".dll"
#  endif
#endif


#define KSI_NORMAL_EXIT_CODE	0
#define KSI_ERROR_EXIT_CODE	1


/*
 * KSI_BOOT_FILE	File that loaded at first stage of startup
 * KSI_KLOS_FILE	Init file for klos
 * KSI_MODULES_FILE	Init file for module system
 * KSI_INIT_FILE	File that loaded at second stage of startup
 */

#ifndef KSI_BOOT_FILE
# define KSI_BOOT_FILE "Boot"
#endif

#ifndef KSI_KLOS_FILE
# define KSI_KLOS_FILE "klos"
#endif

#ifndef KSI_MODULES_FILE
# define KSI_MODULES_FILE "modules"
#endif

#ifndef KSI_INIT_FILE
# define KSI_INIT_FILE "Init"
#endif


/* -------------------------------------------------------------------
 * Some platform characteristics
 *
 * Should be defined in ANSI <limits.h> or in <config.h> by confugure script.
 * If not do guess.
 */

#ifndef CHAR_BIT
#  define CHAR_BIT 8
#endif

#ifndef SIZEOF_SHORT
# define SIZEOF_SHORT 2
#endif

#ifndef SIZEOF_INT
# define SIZEOF_INT 4
#endif

#ifndef SIZEOF_LONG
# define SIZEOF_LONG 4
#endif

#ifndef SIZEOF_VOIDP
# define SIZEOF_VOIDP 4
#endif

#define KSI_INT_SIZE (SIZEOF_INT * CHAR_BIT)

#if (KSI_INT_SIZE == 32)
#  define KSI_INT_DIGS (12)
#  define KSI_INT_SIGN (0x80000000)
#  define KSI_INT_MASK (0xffffffff)
#elif (KSI_INT_SIZE == 64)
#  define KSI_INT_DIGS (22)
#  define KSI_INT_SIGN (0x8000000000000000)
#  define KSI_INT_MASK (0xffffffffffffffff)
#else
#  error "unsupported integer size"
#endif


/*
 * MAX_PATH_LENGTH
 *	Maximal length of path name.
 *	On Posix platform (ie unix) _POSIX_PATH_MAX must be specified.
 *	Some compilers (as Watcom C) specifies _MAX_PATH,
 *	and as last 260 used (this value valid for ms-dos).
 */

#ifdef _POSIX_PATH_MAX
# define MAX_PATH_LENGTH _POSIX_PATH_MAX
#else
# if defined(_MAX_PATH)
#  define MAX_PATH_LENGTH _MAX_PATH
# else
#  define MAX_PATH_LENGTH 260
# endif
#endif


#if defined(unix) || defined(__CYGWIN32__) || defined(__CYGWIN__)

# define ABSOLUTE_PATH(p)			\
  (p[0] == '/' ||				\
   (p[0] == '.' && p[1] == '/') ||		\
   (p[0] == '.' && p[1] == '.' && p[2] == '/'))

# define IS_DIR_SUFFIX(c)	((c) == '/')
# define DIR_SEP		"/"
# define PATH_SEP               ":"

#elif defined(MSDOS) || defined(MSWIN32) || defined(OS2)

# define ABSOLUTE_PATH(p)                                               \
  ((p[0] == '/' || p[0] == '\\') ||                                     \
   p[0] == '.' && (p[1] == '/' || p[1] == '\\') ||                      \
   p[0] == '.' && p[1] == '.' && (p[2] == '/' || p[2] == '\\') ||       \
   p[0] != '\0' && p[1] == ':' && (p[2] == '/' || p[2] == '\\'))

# define IS_DIR_SUFFIX(c)	((c) == '/' || (c) == '\\')
# define DIR_SEP		"\\"
# define PATH_SEP               ";"

#else

# define ABSOLUTE_PATH(p) (0)
# define IS_DIR_SUFFIX(c) (0)

#endif


/*
 * KSI_STACK_GROWS_UP
 *	Direction of stack growing.
 *	Normaly defined in <config.h> by configure script.
 */

#ifndef HAVE_CONFIG_H
# ifndef KSI_STACK_GROWS_UP
#  if defined(hp9000s800) || defined(pyr) || defined(nosve) || defined(_UNICOS)
#   define KSI_STACK_GROWS_UP 1
#  endif
# endif
#endif


#endif


/* End of file */
