/*
 * ksi_prim.h
 * primitive functions
 *
 * Copyright (C) 1997-2000,2001, Ivan Demakov.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without a written agreement
 * is hereby granted, provided that the above copyright notice and this
 * paragraph and the following two paragraphs appear in all copies.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE AUTHORS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *
 * Author:        Ivan Demakov <demakov@users.sourceforge.net>
 * Creation date: Thu Nov 27 03:24:35 1997
 * Last Update:   Sun Mar 25 04:25:40 2001
 *
 * 
 * $Id: ksi_prim.h,v 1.3 2001/03/27 16:42:19 demakov Exp $
 *
 */

#ifndef KSI_PRIM_H
#define KSI_PRIM_H

typedef enum
{
  KSI_CALL_ARG0,
  KSI_CALL_ARG1,
  KSI_CALL_ARG2,
  KSI_CALL_ARG3,
  KSI_CALL_ARG4,
  KSI_CALL_ARG5,
  KSI_CALL_ARG6,
  KSI_CALL_ARG7,
  KSI_CALL_ARG8,
  KSI_CALL_ARG9,
  KSI_CALL_NUM,

  KSI_CALL_REST0 = KSI_CALL_NUM,
  KSI_CALL_REST1,
  KSI_CALL_REST2,
  KSI_CALL_REST3,
  KSI_CALL_REST4,
  KSI_CALL_REST5,
  KSI_CALL_REST6,
  KSI_CALL_REST7,
  KSI_CALL_REST8,
  KSI_CALL_REST9,
  KSI_CALL_LAST = KSI_CALL_REST9
} ksi_call_t;


typedef ksi_obj (*ksi_proc_t) ();

typedef ksi_obj (*ksi_call_arg0) (void);
typedef ksi_obj (*ksi_call_arg1) (ksi_obj a1);
typedef ksi_obj (*ksi_call_arg2) (ksi_obj a1, ksi_obj a2);
typedef ksi_obj (*ksi_call_arg3) (ksi_obj a1, ksi_obj a2, ksi_obj a3);
typedef ksi_obj (*ksi_call_arg4) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				  ksi_obj a4);
typedef ksi_obj (*ksi_call_arg5) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				  ksi_obj a4, ksi_obj a5);
typedef ksi_obj (*ksi_call_arg6) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				  ksi_obj a4, ksi_obj a5, ksi_obj a6);
typedef ksi_obj (*ksi_call_arg7) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				  ksi_obj a4, ksi_obj a5, ksi_obj a6,
				  ksi_obj a7);
typedef ksi_obj (*ksi_call_arg8) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				  ksi_obj a4, ksi_obj a5, ksi_obj a6,
				  ksi_obj a7, ksi_obj a8);
typedef ksi_obj (*ksi_call_arg9) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				  ksi_obj a4, ksi_obj a5, ksi_obj a6,
				  ksi_obj a7, ksi_obj a8, ksi_obj a9);

typedef ksi_obj (*ksi_call_rest0) (int ac, ksi_obj *av);
typedef ksi_obj (*ksi_call_rest1) (ksi_obj a1,
				   int ac, ksi_obj *av);
typedef ksi_obj (*ksi_call_rest2) (ksi_obj a1, ksi_obj a2,
				   int ac, ksi_obj *av);
typedef ksi_obj (*ksi_call_rest3) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				   int ac, ksi_obj *av);
typedef ksi_obj (*ksi_call_rest4) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				   ksi_obj a4,
				   int ac, ksi_obj *av);
typedef ksi_obj (*ksi_call_rest5) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				   ksi_obj a4, ksi_obj a5,
				   int ac, ksi_obj *av);
typedef ksi_obj (*ksi_call_rest6) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				   ksi_obj a4, ksi_obj a5, ksi_obj a6,
				   int ac, ksi_obj *av);
typedef ksi_obj (*ksi_call_rest7) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				   ksi_obj a4, ksi_obj a5, ksi_obj a6,
				   ksi_obj a7,
				   int ac, ksi_obj *av);
typedef ksi_obj (*ksi_call_rest8) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				   ksi_obj a4, ksi_obj a5, ksi_obj a6,
				   ksi_obj a7, ksi_obj a8,
				   int ac, ksi_obj *av);
typedef ksi_obj (*ksi_call_rest9) (ksi_obj a1, ksi_obj a2, ksi_obj a3,
				   ksi_obj a4, ksi_obj a5, ksi_obj a6,
				   ksi_obj a7, ksi_obj a8, ksi_obj a9,
				   int ac, ksi_obj *av);

struct Ksi_Prim_Def
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  const char	*name;	/* name of primitive */
  ksi_proc_t	proc;	/* function */
  ksi_call_t	call;	/* call type */
  int		reqv;	/* number of required args */
};


struct Ksi_Prim
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned		itag;
  ksi_proc_t		proc;
  ksi_call_t		call;
  int			reqv;
  const char		*name;
};

#define KSI_PRIM_PROC(x)	(((ksi_prim) (x)) -> proc)
#define KSI_PRIM_CALL(x)	(((ksi_prim) (x)) -> call)
#define KSI_PRIM_REQV(x)	(((ksi_prim) (x)) -> reqv)
#define KSI_PRIM_NAME(x)	(((ksi_prim) (x)) -> name)


typedef struct Ksi_Prim_Closure
{
  int			itag;
  int			argc;
  ksi_obj		prim;
  ksi_obj		argv[1];
} *ksi_prim_closure;


#ifdef __cplusplus
extern "C" {
#endif

SI_API
ksi_obj
ksi_defun (const char *name, ksi_proc_t proc, ksi_call_t call, int reqv);

SI_API
ksi_obj
ksi_reg_prim (struct Ksi_Prim_Def* prim);

SI_API
void
ksi_reg_unit (struct Ksi_Prim_Def* prims);


  /* depricated API */

SI_API
ksi_obj
ksi_make_prim (ksi_proc_t proc, void *data, const char *name);


#ifdef __cplusplus
}
#endif

#endif

 /* End of file */
