/*
 * ksi_type.h
 * public type, constant and macro
 *
 * Copyright (C) 1997-2000,2001, ivan demakov
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without a written agreement
 * is hereby granted, provided that the above copyright notice and this
 * paragraph and the following two paragraphs appear in all copies.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE AUTHORS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *
 * Author:        Ivan Demakov <demakov@users.sourceforge.net>
 * Creation date: Mon Dec 15 12:03:22 1997
 * Last Update:   Sun Nov 18 01:18:43 2001
 *
 * 
 * $Id: ksi_type.h,v 1.18 2001/11/25 14:30:01 demakov Exp $
 *
 */

#ifndef KSI_TYPE_H
#define KSI_TYPE_H

#ifndef KSI_H
#  include "ksi.h"
#endif


enum ksi_tc_tag
{
  KSI_TAG_FIRST_CONST,
  KSI_TAG_SYMBOL = KSI_TAG_FIRST_CONST,
  KSI_TAG_PAIR,
  KSI_TAG_CONST_PAIR,
  KSI_TAG_VECTOR,
  KSI_TAG_CONST_VECTOR,
  KSI_TAG_STRING,
  KSI_TAG_CONST_STRING,
  KSI_TAG_KEYWORD,
  KSI_TAG_BIGNUM,
  KSI_TAG_FLONUM,
  KSI_TAG_RATIONAL,
  KSI_TAG_COMPLEX,
  KSI_TAG_LAST_CONST = KSI_TAG_COMPLEX,

  KSI_TAG_VAR0,
  KSI_TAG_VAR1,
  KSI_TAG_VAR2,
  KSI_TAG_VARN,

  KSI_TAG_FREEVAR,
  KSI_TAG_LOCAL,
  KSI_TAG_IMPORTED,
  KSI_TAG_GLOBAL,

  KSI_TAG_FIRST_CODE,
  KSI_TAG_QUOTE = KSI_TAG_FIRST_CODE,
  KSI_TAG_BEGIN,
  KSI_TAG_AND,
  KSI_TAG_OR,
  KSI_TAG_IF2,
  KSI_TAG_IF3,
  KSI_TAG_DELAY,
  KSI_TAG_LAMBDA,
  KSI_TAG_DEFINE,
  KSI_TAG_SET,
  KSI_TAG_CALL,
  KSI_TAG_FRAME,
  KSI_TAG_LAST_CODE = KSI_TAG_FRAME,

  KSI_TAG_FIRST_OP,
  KSI_TAG_APPLY = KSI_TAG_FIRST_OP,
  KSI_TAG_CLOSE, /* new */
  KSI_TAG_CALL_CC,
  KSI_TAG_CALL_WITH_VALUES,
  KSI_TAG_NOT,
  KSI_TAG_EQP,
  KSI_TAG_EQVP,
  KSI_TAG_EQUALP,
  KSI_TAG_MEMQ,
  KSI_TAG_MEMV,
  KSI_TAG_MEMBER,
  KSI_TAG_CONS,
  KSI_TAG_CAR,
  KSI_TAG_CDR,
  KSI_TAG_LIST,
  KSI_TAG_MAKE_VECTOR, /* new */
  KSI_TAG_NULLP,
  KSI_TAG_PAIRP,
  KSI_TAG_LISTP,
  KSI_TAG_VECTORP,
  KSI_TAG_ADD, /* new */
  KSI_TAG_SUB, /* new */
  KSI_TAG_MUL, /* new */
  KSI_TAG_DIV, /* new */
  KSI_TAG_LAST_OP = KSI_TAG_DIV,

  KSI_TAG_FIRST_PROC,
  KSI_TAG_PRIM = KSI_TAG_FIRST_PROC,
  KSI_TAG_PRIM_0,
  KSI_TAG_PRIM_1,
  KSI_TAG_PRIM_2,
  KSI_TAG_PRIM_r,
  KSI_TAG_FUNC,
  KSI_TAG_CLOSURE,
  KSI_TAG_PRIM_CLOSURE,
  KSI_TAG_INSTANCE,
  KSI_TAG_NEXT_METHOD,
  KSI_TAG_LAST_PROC = KSI_TAG_NEXT_METHOD,

  KSI_TAG_FIRST_RUNTIME,
  KSI_TAG_PORT = KSI_TAG_FIRST_RUNTIME,
  KSI_TAG_CORE, /* new */
  KSI_TAG_PROMISE,
  KSI_TAG_VALUES,
  KSI_TAG_HASHTAB,
  KSI_TAG_ENVIRON,
  KSI_TAG_MODULE,
  KSI_TAG_POINTER,
  KSI_TAG_CONST_POINTER,
  KSI_TAG_ANNOTATION,
  KSI_TAG_EXN,
  KSI_TAG_EVENT,
  KSI_TAG_LAST_RUNTIME = KSI_TAG_EVENT,

  KSI_TAG_EXTENDED,
  KSI_TAG_BROKEN
};


/*
 * WARNING:
 *   The order of fields in the structures cannot be changed without
 *   corresponding changes in the code that allocate memory for the objects.
 *   Ksi library uses the typed allocation (see `gc/gc_typed.h'),
 *   and library should know the offsets of pointers and none-pointers
 *   in the structure that allocated.
 */

struct Ksi_Tag
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  const char*	type_name;

  int		(*equal) (ksi_tag, ksi_obj x1, ksi_obj x2, int deep);
  const char*	(*print) (ksi_tag, ksi_obj x, int slashify);
};

struct Ksi_Obj
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
};


struct Ksi_Symbol
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  int		len;
  const char*	ptr;
};

struct Ksi_Keyword
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  int		len;
  const char*	ptr;
};

struct Ksi_String
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  int		len;
  char*		ptr;
};

struct Ksi_Pair
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  ksi_obj	car;
  ksi_obj	cdr;
};

struct Ksi_Vector
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  int		dim;
  ksi_obj	arr [1];
};

struct Ksi_Bignum
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc_data (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  int       	sign;
  unsigned	digs[1];
};

struct Ksi_Flonum
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc_data (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  double	real;
};

struct Ksi_Complex
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc_data (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  double	real;
  double	imag;
};

struct Ksi_Pointer
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  size_t	size;
  void*		val;
};

struct Ksi_ExtObj
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  ksi_tag	etag;
};


struct Ksi_Port_Tag
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  const char*	(*name) (ksi_port);
  int		(*close) (ksi_port);
  int		(*read) (ksi_port, char* buf, int buf_len);
  int		(*write) (ksi_port, const char* buf, int buf_len);
  int		(*input_ready) (ksi_port);
  int		(*output_ready) (ksi_port);
  int		(*flush) (ksi_port);
  int		(*input_fd) (ksi_port);
  int		(*output_fd) (ksi_port);
};

struct Ksi_Port
{
#ifdef __cplusplus
  void* operator new (size_t sz) { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned	itag;
  ksi_port_tag	ops;
  int		read_line, read_pos;
  char		last_write_char;
  char		unread_num;
  char		unread_chars[6];

  unsigned	input	: 1;
  unsigned	output	: 1;
  unsigned	closed	: 1;
  unsigned	eof	: 1;

  unsigned	unbuf   : 1;
  unsigned	linebuf : 1;
  unsigned	async   : 1;

  unsigned	is_fd   : 1;
  unsigned	is_tty  : 1;
  unsigned	is_ext  : 1;

  unsigned	is_dir  : 1;
  unsigned	is_chr  : 1;
  unsigned	is_blk  : 1;
  unsigned	is_reg  : 1;
  unsigned	is_fifo : 1;
  unsigned	is_lnk  : 1;
  unsigned	is_sock : 1;
  unsigned	is_inet : 1;
};


struct Ksi_Event_Mgr
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  void  (*init) (ksi_event_mgr mgr);
  void  (*term) (ksi_event_mgr mgr);

  void*	(*wait_timer) (ksi_event_mgr mgr, ksi_event evt, double tm, int restart);
  void	(*cancel_timer) (ksi_event_mgr mgr, ksi_event evt, void *data);

  void*	(*wait_input) (ksi_event_mgr mgr, ksi_event evt, int fd, int restart);
  void	(*cancel_input) (ksi_event_mgr mgr, ksi_event evt, void *data);

  void*	(*wait_output) (ksi_event_mgr mgr, ksi_event evt, int fd, int restart);
  void	(*cancel_output) (ksi_event_mgr mgr, ksi_event evt, void *data);

  void*	(*wait_signal) (ksi_event_mgr mgr, ksi_event evt, int sig, int restart);
  void	(*cancel_signal) (ksi_event_mgr mgr, ksi_event evt, void *data);

  void*	(*wait_idle) (ksi_event_mgr mgr, ksi_event evt, int restart);
  void	(*cancel_idle) (ksi_event_mgr mgr, ksi_event evt, void *data);

  void  (*wait_event) (ksi_event_mgr mgr, double tm);

  void  (*enable_async_wait) (ksi_event_mgr mgr);
  void  (*disable_async_wait) (ksi_event_mgr mgr);

  void  (*block_wait) (ksi_event_mgr mgr);
  void  (*unblock_wait) (ksi_event_mgr mgr);
};

struct Ksi_Event_Tag
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  const char*	(*name) (ksi_event);
  void		(*init) (ksi_event);
  void		(*setup) (ksi_event);
  void		(*cancel) (ksi_event);
  int		(*invoke) (ksi_event, void *data);
};

struct Ksi_Event
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  int		itag;
  ksi_event_tag	ops;
  ksi_obj	state;
  ksi_obj	action;
  ksi_obj	result;

  ksi_event	next, prev;
  void		*data;

  unsigned	pending : 1;
  unsigned	waiting : 1;
  unsigned	active: 1;
  unsigned	ready : 1;
  unsigned	inited : 1;
  unsigned	start : 1;
  unsigned	stop : 1;
};


#define KSI_OBJ_TAG(c)  (((ksi_obj) (c)) -> itag)
#define KSI_EXT_TAG(c)  (((ksi_extobj) c) -> etag)

#define KSI_OBJ_IS(c,t)	(KSI_OBJ_P (c) && (KSI_OBJ_TAG (c) == (t)))
#define KSI_EXT_IS(c,t) (KSI_OBJ_IS ((c), KSI_TAG_EXTENDED) \
			 && KSI_EXT_TAG (c) == (t))

/*
 * KSI_SYM_P
 *	    .
 *
 * KSI_SYM_PTR
 *	  (char*)   .
 *
 * KSI_SYM_LEN
 *	   .
 *
 */

#define KSI_SYM_P(s)		(KSI_OBJ_IS ((s), KSI_TAG_SYMBOL))
#define KSI_SYM_PTR(s)		(((ksi_symbol) (s)) -> ptr)
#define KSI_SYM_LEN(s)		(((ksi_symbol) (s)) -> len)


/*
 * KSI_KEY_P
 *	     
 *
 * KSI_KEY_PTR
 *	     
 *
 * KSI_KEY_LEN
 *	    
 */

#define KSI_KEY_P(s)		(KSI_OBJ_IS ((s), KSI_TAG_KEYWORD))
#define KSI_KEY_PTR(s)		(((ksi_keyword) (s)) -> ptr)
#define KSI_KEY_LEN(s)		(((ksi_keyword) (s)) -> len)


/*
 * KSI_BIGNUM_P
 *	     .
 *
 * KSI_FLONUM_P
 *	       .
 *
 * KSI_COMPLEX_P
 *	     .
 *
 * KSI_EXACT_P
 *	     .
 *
 * KSI_INEXACT_P
 *	     .
 *
 * KSI_NUM_P
 *	    .
 *
 * KSI_EINT_P
 *	      .
 */

#define KSI_BIGNUM_P(x)		(KSI_OBJ_IS ((x), KSI_TAG_BIGNUM))
#define KSI_FLONUM_P(x)		(KSI_OBJ_IS ((x), KSI_TAG_FLONUM))
#define KSI_COMPLEX_P(x)	(KSI_OBJ_IS ((x), KSI_TAG_COMPLEX))

#define KSI_EINT_P(x)		(KSI_SINT_P (x) || KSI_BIGNUM_P (x))
#define KSI_REAL_P(x)		(KSI_EINT_P(x) || KSI_FLONUM_P (x))
#define KSI_EXACT_P(o)		(KSI_SINT_P (o) || KSI_BIGNUM_P (o))
#define KSI_INEXACT_P(o)	(KSI_FLONUM_P (o) || KSI_COMPLEX_P (o))
#define KSI_NUM_P(x)		(KSI_EXACT_P(x) || KSI_INEXACT_P (x))

#define KSI_REPART(x)		(((ksi_flonum) (x)) -> real)
#define KSI_IMPART(x)		(((ksi_complex) (x)) -> imag)


/*
 * KSI_STR_P
 *	    .
 *
 * KSI_C_STR_P
 *	     .
 *
 * KSI_STR_PTR
 *	    .
 *	       .
 *	    ,   
 *	     .
 *
 * KSI_STR_LEN
 *	  
 */

#define KSI_C_STR_P(s)		(KSI_OBJ_IS ((s), KSI_TAG_CONST_STRING))
#define KSI_M_STR_P(s)		(KSI_OBJ_IS ((s), KSI_TAG_STRING))
#define KSI_STR_P(s)		(KSI_C_STR_P (s) || KSI_M_STR_P (s))
#define KSI_STR_PTR(o)		(((ksi_string) (o)) -> ptr)
#define KSI_STR_LEN(o)		(((ksi_string) (o)) -> len)


/*
 * KSI_NULL_P
 *	     
 *
 * KSI_PAIR_P
 *	    
 *
 * KSI_PLIST_P
 *	      
 *
 * KSI_LIST_P
 *	     
 *
 * KSI_LIST_LEN
 *	  .
 *	    ,  
 *	.
 */

#define KSI_NULL_P(x)	((x) == ksi_nil)
#define KSI_M_PAIR_P(s)	KSI_OBJ_IS ((s), KSI_TAG_PAIR)
#define KSI_C_PAIR_P(s)	KSI_OBJ_IS ((s), KSI_TAG_CONST_PAIR)
#define KSI_PAIR_P(x)	(KSI_M_PAIR_P (x) || KSI_C_PAIR_P (x))
#define KSI_PLIST_P(x)	(ksi_list_len ((x)) > 0)
#define KSI_LIST_P(x)	((x) == ksi_nil || KSI_PLIST_P(x))
#define KSI_LIST_LEN(x) (ksi_list_len ((x)))


/*
 * KSI_CAR
 *	  car 
 *
 * KSI_CDR
 *	  cdr 
 */

#define KSI_CAR(x)	(((ksi_pair) (x)) -> car)
#define KSI_CDR(x)	(((ksi_pair) (x)) -> cdr)


/*
 * KSI_VEC_P
 *	    
 *
 * KSI_VEC_LEN
 *	  
 *
 * KSI_VEC_REF
 *	  
 */

#define KSI_M_VEC_P(x)		KSI_OBJ_IS ((x), KSI_TAG_VECTOR)
#define KSI_C_VEC_P(x)		KSI_OBJ_IS ((x), KSI_TAG_CONST_VECTOR)
#define KSI_VEC_P(x)		(KSI_M_VEC_P (x) || KSI_C_VEC_P (x))
#define KSI_VEC_LEN(x)		(((ksi_vector) (x)) -> dim)
#define KSI_VEC_ARR(x)		(((ksi_vector) (x)) -> arr)
#define KSI_VEC_REF(x,n)	(((ksi_vector) (x)) -> arr [(n)])


/*
 * KSI_PORT_P
 *	    
 */

#define KSI_PORT_P(x)		(KSI_OBJ_IS (x, KSI_TAG_PORT))
#define KSI_INPUT_PORT_P(x)	(KSI_PORT_P (x) \
				 && (((ksi_port) (x)) -> input))
#define KSI_OUTPUT_PORT_P(x)	(KSI_PORT_P (x) \
				 && (((ksi_port) (x)) -> output))
#define KSI_CLOSED_PORT_P(x)	(KSI_PORT_P (x) \
				 && (((ksi_port) (x)) -> closed))
#define KSI_STD_PORT_P(x)	(KSI_PORT_P (x) \
				 && (((ksi_port) (x)) -> std))
#define KSI_TTY_PORT_P(x)	(KSI_PORT_P (x) \
				 && (((ksi_port) (x)) -> tty))

#define KSI_M_CPTR_P(x)		(KSI_OBJ_IS ((x), KSI_TAG_POINTER))
#define KSI_C_CPTR_P(x)		(KSI_OBJ_IS ((x), KSI_TAG_CONST_POINTER))
#define KSI_CPTR_P(x)		(KSI_C_CPTR_P(x) || KSI_M_CPTR_P(x))
#define KSI_CPTR_VAL(x)         (((ksi_pointer) (x)) -> val)
#define KSI_CPTR_SZ(x)          (((ksi_pointer) (x)) -> size)

#define KSI_EXN_P(x)		(KSI_OBJ_IS ((x), KSI_TAG_EXN))
#define KSI_EXN_TYPE(x)		KSI_VEC_REF ((x), 0)
#define KSI_EXN_MSG(x)		KSI_VEC_REF ((x), 1)
#define KSI_EXN_VAL(x)		KSI_VEC_REF ((x), 2)
#define KSI_EXN_PRI(x)		KSI_VEC_REF ((x), 3)

#define KSI_EVT_P(x)		(KSI_OBJ_IS ((x), KSI_TAG_EVENT))
#define KSI_EVT_STATE(x)	(((ksi_event) (x)) -> state)
#define KSI_EVT_ACTION(x)	(((ksi_event) (x)) -> action)
#define KSI_EVT_RESULT(x)	(((ksi_event) (x)) -> result)

#define KSI_PRIM_P(x)		(KSI_OBJ_IS ((x), KSI_TAG_PRIM)		\
				 || KSI_OBJ_IS ((x), KSI_TAG_PRIM_0)	\
				 || KSI_OBJ_IS ((x), KSI_TAG_PRIM_1)	\
				 || KSI_OBJ_IS ((x), KSI_TAG_PRIM_2)	\
				 || KSI_OBJ_IS ((x), KSI_TAG_PRIM_r)

#define KSI_NEW_TYPED(t,f,l)					\
  ((t*) ksi_malloc_typed (sizeof (t),				\
			  offsetof (t, f), offsetof (t, l)))

#define KSI_NEW_TYPED_EXT(t,s,f,l)				\
  ((t*) ksi_malloc_typed (sizeof (t) + (s),			\
			  offsetof (t, f), offsetof (t, l)))

#define KSI_NEW_TYPED_ARR(t,n,f,l)					\
  ((t*) ksi_malloc_typed (sizeof (t) + ((n) - 1) * sizeof (ksi_obj),	\
			  offsetof (t, f), offsetof (t, l [(n) - 1])))


#ifdef __cplusplus
extern "C" {
#endif

SI_API
ksi_obj
ksi_abbrev (char* str, int len);

SI_API
int
ksi_default_tag_equal (ksi_tag, ksi_obj x1, ksi_obj x2, int deep);

SI_API
const char*
ksi_default_tag_print (ksi_tag, ksi_obj x, int slashify);

SI_API
ksi_obj
ksi_make_fd_port (int fd, const char *fname, const char *mode);

SI_API
ksi_obj
ksi_open_fd_port (const char *fname, const char *mode);

SI_API
int
ksi_port_putc (ksi_port port, int ch);

SI_API
int
ksi_port_getc (ksi_port port);

SI_API
void
ksi_port_ungetc (ksi_port port, int ch);

SI_API
int
ksi_port_read (ksi_port port, char* ptr, int len);

SI_API
int
ksi_port_write (ksi_port port, const char* ptr, int len);

SI_API
ksi_event_mgr
ksi_register_event_mgr (ksi_event_mgr mgr);

SI_API
ksi_event_mgr
ksi_current_event_mgr (void);

SI_API
void*
ksi_wait_timer (ksi_event evt, double tm, int restart);

SI_API
void
ksi_cancel_timer (ksi_event evt, void *data);

SI_API
void*
ksi_wait_input (ksi_event evt, int fd, int restart);

SI_API
void
ksi_cancel_input (ksi_event evt, void *data);

SI_API
void*
ksi_wait_output (ksi_event evt, int fd, int restart);

SI_API
void
ksi_cancel_output (ksi_event evt, void *data);

SI_API
void*
ksi_wait_signal (ksi_event evt, int sig, int restart);

SI_API
void
ksi_cancel_signal (ksi_event evt, void *data);

SI_API
void*
ksi_wait_idle (ksi_event evt, int restart);

SI_API
void
ksi_cancel_idle (ksi_event evt, void *data);

SI_API
void
ksi_run_event (ksi_event evt, void *data, int invoke);

SI_API
int
ksi_run_pending_events (void);

SI_API
int
ksi_has_pending_events (void);


#ifdef __cplusplus
}
#endif

#endif

 /* End of file */
