/* -*-mode:C++-*- */
/*
 * ksi_vtab.h
 * string table
 *
 * Copyright (C) 1997-2000, Ivan Demakov.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without a written agreement
 * is hereby granted, provided that the above copyright notice and this
 * paragraph and the following two paragraphs appear in all copies.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE AUTHORS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *
 * Author:        Ivan Demakov <demakov@users.sourceforge.net>
 * Creation date: Fri Dec  5 04:39:08 1997
 * Last Update:   Tue Mar 21 00:13:31 2000
 *
 * 
 * $Id: ksi_vtab.h,v 1.1.1.1 2000/07/25 19:14:45 demakov Exp $
 *
 */

#ifndef KSI_VTAB_H
#define KSI_VTAB_H


typedef unsigned (*ksi_hash_fun) (void* val, unsigned size, void* data);
typedef int (*ksi_cmp_fun) (void* v1, void* v2, void* data);
typedef int (*ksi_iter_fun) (void* val, void* iter_data);


struct Ksi_Tabrec
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  struct Ksi_Tabrec	*next;
  void			*val;
};

struct Ksi_Valtab
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  struct Ksi_Tabrec**	table;
  unsigned		size;	/* table size */
  int			count;	/* number of recs */

  ksi_hash_fun		hash;
  ksi_cmp_fun		cmp;
  void*			data;
};


#ifdef __cplusplus
extern "C" {
#endif

SI_API
ksi_valtab
ksi_new_valtab (unsigned init_size,
		ksi_hash_fun hash, ksi_cmp_fun cmp, void* data);

SI_API
void
ksi_clear_vtab (ksi_valtab tab);

SI_API
void*
ksi_lookup_vtab (ksi_valtab tab, void* val);

SI_API
void
ksi_append_vtab (ksi_valtab tab, void* val);

SI_API
void*
ksi_remove_vtab (ksi_valtab tab, void* val);

SI_API
void*
ksi_iterate_vtab (ksi_valtab tab, ksi_iter_fun fun, void* iter_data);

SI_API
void
ksi_rebuild_vtab (ksi_valtab tab, unsigned);


SI_API
ksi_valtab
ksi_new_symtab (unsigned init_size);

SI_API
ksi_symbol
ksi_lookup_sym (ksi_valtab tab, const char* key, int len);

SI_API
void
ksi_append_sym (ksi_valtab tab, ksi_symbol sym);

SI_API
ksi_obj
ksi_abbrev_sym (ksi_valtab tab, const char* key, int len);


SI_API
ksi_valtab
ksi_new_keytab (unsigned init_size);

SI_API
ksi_keyword
ksi_lookup_key (ksi_valtab tab, const char* key, int len);

SI_API
void
ksi_append_key (ksi_valtab tab, ksi_keyword key);

SI_DATA
unsigned ksi_primes [];

#ifdef __cplusplus
}
#endif


#endif

 /* End of file */
